
## This directory should be changed  
source('C:/.../SourceFunctions.R')


install.packages('survey')
library(survey)
 
set.seed(1455)


N <- 100000
n<-  1000

# This creates an outcome such that E(Y) = logit( -1- X1 - 0.5 Z1 )- X1~ ber(logit(0.5+Z1  )) ;  X2~ ber(0.5) ; Z1~ber(0.5)
# There are 100 clusters in the full data, each of size 100 
# The sample is an stratified balanced sample of size 1000, where  strata are defined on the basis of outcome and X1.  
# The following code generate the variables
 
X1 <- rbinom(N, 1, 0.2) 
X2 <- rbinom(N, 1, 0.3) 
Z1 <- rbinom(N, 1, 0.3) 
#The following generated the outcome variable and induces correlation within clusters 
K   <- 100
id  <- sort( rep(1:K, 1000))
ids <- sort(unique( id))
sigmaV   <- 0.5
pi.Y     <- numeric()
Vk       <- rnorm(K, 0,sigmaV)
for(k in 1:K) pi.Y[id == ids[k]] <-  1/( 1+exp( 1+X1[id==ids[k]] +X2[id==ids[k]]+0.5*Z1[id==ids[k]] + Vk[k]))     

Y <- rbinom(N, 1, pi.Y)
 mean(Y)
 
# Now, we  select the sample and calculate the sample weights
samp  <- c(sample((1:N)[ Y==1 & X1==1 ] , n/4))
samp  <- c(samp, c(sample((1:N)[ Y==0  & X1==1 ] , n/4)) )
samp  <- c(samp, c(sample((1:N)[ Y==0 & X1==0 ] , n/4)) )
samp  <- c(samp, c(sample((1:N)[ Y==1 & X1==0 ] , n/4)) )
SampInd <- as.numeric( 1: N %in% samp)
nh   <-  rep(n/4, N)
Nh   <-  rep(1, N)
Nh[ Y==1 & X1==1   ] <- sum(  Y==1 & X1==1  ) 
Nh[ Y==1 & X1==0   ] <- sum(  Y==1 & X1==0  ) 
Nh[ Y==0 & X1==1   ] <- sum(  Y==0 & X1==1  ) 
Nh[ Y==0 & X1==0   ] <- sum(  Y==0 & X1==0  ) 
wgt  <-  Nh/nh 


##

# Now we form a dataset with all the information and a dataset with the sample information 
Data.Full <- data.frame(Y, X1, X2, Z1, wgt, SampInd, id, idInd=1:N, nh, Nh)
Data.Samp <- Data.Full[Data.Full$SampInd==1, ]
 
## This calculated the second phase inclustion probabilitirs and 


## Ordering the data according to cluster id 
Data.Samp <- Data.Samp[order(Data.Samp$id), ] 
mySSS     <- paste( 'S', Data.Samp$Y, Data.Samp$X1 , sep='')    
Delta     <- DeltaComp(mySSS , Data.Samp$nh , Data.Samp$Nh )
Deltaij   <- Delta[[2]]
PIij      <- Delta[[1]]

formulaS  <- as.formula(Y~  X1+X2 +Z1) 

### With the logit link : 

# This fits the WGEE with independent correlation structure 
fitWGEE_Ind        <-   tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$wgt,   PIij=PIij,    link='logit')
# This fits the WGEE with exchangeable correlation structure 
fitWGEE_Ex        <-   tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$wgt,  PIij=PIij,  w.corr='exchangeable', link='logit')

### With the log link : 

fitWGEE_Ind2        <-   tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$wgt,  PIij=PIij,    link='log')
# This fits the WGEE with exchangeable correlation structure 
fitWGEE_Ex2        <-   tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$wgt,  PIij=PIij,   w.corr='exchangeable', link='log')





### With calibration. In this part we calibrate to the actual variables  X1 and X2 



X_CF           <- data.frame( 'C0'= 1,      'C1'=   Data.Full$Y*Data.Full$X2*Data.Full$X1     ,   'C2'=   Data.Full$X2    ,      'C3'=   Data.Full$Y*Data.Full$X2     ,
                              'C4'=   Data.Full$Y*Data.Full$X1   , 'C5'=   Data.Full$X2*Data.Full$X1      )
X_C            <- data.frame( 'C0'= 1,       'C1'=   Data.Samp$Y*Data.Samp$X2 *Data.Samp$X1  ,     'C2'=   Data.Samp$X2   ,      'C3'=   Data.Samp$Y*Data.Samp$X2     ,
                              'C4'=   Data.Samp$Y*Data.Samp$X1   , 'C5'=   Data.Samp$X2*Data.Samp$X1      )
totv           <- c(colSums( X_CF))
formcal        <- formula('~      C1+   C2 +C3+C4+C5')
de1            <- svydesign(~1, probs=1/Data.Samp$wgt ,  data = X_C ) 
cal1           <- calibrate(de1, formcal , as.vector(totv) , calfun='raking'  ) 
 
 
weightsc  <-       1/cal1$prob 
Data.Samp$weightsc <-weightsc 
plot( Data.Samp$weightsc ,Data.Samp$wgt)
 

### With the logit link : 

# This fits the WGEE with independent correlation structure 
fitWGEE_Ind_cal    <- tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$weightsc ,  PIij=PIij ,   link='logit', X_C = X_C, calib=TRUE , weightsDes=Data.Samp$wgt )

# This fits the WGEE with exchangeable correlation structure 
fitWGEE_Exc_cal    <- tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$weightsc,   PIij=PIij,   link='logit', X_C = X_C, calib=TRUE , weightsDes=Data.Samp$wgt, w.corr='exchangeable')

### With the log  link : 

# This fits the WGEE with independent correlation structure 
fitWGEE_Ind_cal2    <- tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$weightsc, PIij=PIij,    link='log', X_C = X_C, calib=TRUE , weightsDes=Data.Samp$wgt)
# This fits the WGEE with exchangeable correlation structure 
fitWGEE_Exc_cal2    <- tpsWGEE(formulaS, data=Data.Samp, id=Data.Samp$id, weights=Data.Samp$weightsc, PIij=PIij,     link='log', X_C = X_C, calib=TRUE , weightsDes=Data.Samp$wgt, w.corr='exchangeable')

?geeglm
library(geepack)






